J = 25; %Number of grid points (in x and y directions)
[x, y] = meshgrid(0:1/(J+1):1); %values of (x,y) at grid points
dx = 1/(J+1); %Spatial step
dy = 1/(J+1);
h = dx;  %Call dx=h
err = h^2; %Error tolerance at which iterations stop
            %Since finite differences are O(h^2) accurate
            %there is no need to solve scheme at an accuracy beyond h^2

f = 100*(x - y);  %Many options for f
g = x.^2 + y.^2; %Boundary values
u0 = g;
u0(2:end-1,2:end-1) = zeros(J,J);  %Initial guess is zero matrix

zmin = min(u0(:));
zmax = max(u0(:));

u=u0;

surf(x,y,u);
pause;
key_pressed=0;
set(gcf, 'KeyPressFcn', 'key_pressed=mod(key_pressed+1,2);')

e=1;
i=1;

%Press any key to stop/start simulation
while e > err  %Loop until scheme error e is less than err=h^2
    
    i
    uE = u(3:end,2:end-1);  %East grid point
    uW = u(1:end-2,2:end-1);%West grid point
    uS = u(2:end-1,1:end-2);%South grid point
    uN = u(2:end-1,3:end);  %North grid point
    F = f(2:end-1,2:end-1);
    v = (uE + uW + uS + uN + h^2*F)/4;  %Jacobi iteration
    
    %Compute error in solving scheme
    e = max(max(abs(u(2:end-1,2:end-1) -v)))/h^2

    u(2:end-1,2:end-1) = v; %Update interior grid points and leave boundary values
                            %Change this line for Neumann conditions
    %Draw u
    surf(x,y,u);
    zmin=min(min(u(:)),zmin);
    zmax=max(max(u(:)),zmax);
    zlim([zmin zmax]);
    drawnow;
    
    %Pause when key is pressed
    while key_pressed == 1
        pause(0.1);
    end
    i = i+1;
end
